/*!
 * shuffle-letters
 * A JavaScript library to shuffle the text content of a DOM element with an animated effect.
 *
 * @version v1.1.0
 * @author George Raptis <georapbox@gmail.com>
 * @homepage https://github.com/georapbox/shuffle-letters#readme
 * @repository git+https://github.com/georapbox/shuffle-letters.git
 * @license MIT
 */
function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

/**
 * Get a pseudo-random character from an available pool depending on the type of the character.
 *
 * @param {string} charType The type of the character. Available values are "lowerCase", "upperCase" and "symbol".
 * @returns {string} Returns the pseudo-random character.
 */
function getRandomCharacterForType(charType) {
  var charsPool = '';

  switch (charType) {
    case 'lowerCase':
      charsPool = 'abcdefghijklmnopqrstuvwxyz';
      break;

    case 'upperCase':
      charsPool = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
      break;

    case 'digit':
      charsPool = '0123456789';
      break;

    case 'symbol':
      charsPool = ',.?/\\(^)![]{}*&^%$#\'"';
      break;
  }

  var charsPoolArray = charsPool.split('');
  return charsPoolArray[Math.floor(Math.random() * charsPoolArray.length)];
}

/**
 * Checks if a value is a DOM element.
 *
 * @param {*} value The value to check.
 * @return {Boolean} True if 'value' is a DOM element, else false.
 * @example
 *
 * isElement('Lorem ipusm dolr sit amet.');
 * // -> false
 *
 * isElement(document.body);
 * // -> true
 *
 * isElement(document.getElementsByTagName('head')[0])
 * // -> true
 */
function isElement(value) {
  return !!value && value.nodeType === 1 && value instanceof Element === true;
}

/**
 * Shuffle the text content of a DOM element
 *
 * @param {HTMLElement} element The HTML element whose `textContent` we want to apply the shuffle effect.
 * @param {object} [config={}] Configurable object to override the default options.
 * @param {string} [config.text=''] Alternative text that can be used instead of the `textContent` of the `element`.
 * @param {number} [config.iterations=8] The number of times the characters will be shuffled before the animation ends.
 * @param {number} [config.fps=30] The amount of frames per second that the animation runs.
 * @param {function} [config.onComplete=() => void] A callback function that is called when the animation of the effect is complete.
 * @returns {function} Returns a function that when called, it clears the `timeoutID` which identifies the timer created by the call to `setTimeout()` that is used internally for the shuffle effect.
 */

function shuffleLetters(element) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  /**
   * Default options, if not provided by the user.
   * @type {object}
   */
  var defaults = {
    text: '',
    iterations: 8,
    fps: 30,
    onComplete: function onComplete() {
      return void 0;
    }
  };

  var options = _objectSpread2(_objectSpread2({}, defaults), config);

  if (!isElement(element)) {
    throw new TypeError('Expected element to be a valid HTML element.');
  }
  /**
   * Holds all characters of the string to process.
   * @type {string[]}
   */


  var charsArray = options.text && typeof options.text === 'string' ? options.text.split('') : element.textContent.split('');
  /**
   * Holds the type for each character.
   * @type {string[]}
   */

  var charsTypes = [];
  /**
   * Holds the positions of non-space characters.
   * @type {number[]}
   */

  var charsPositions = [];

  for (var i = 0; i < charsArray.length; i += 1) {
    var char = charsArray[i];

    if (/\s/.test(char)) {
      charsTypes[i] = 'space';
      continue;
    } else if (/[a-z]/.test(char)) {
      charsTypes[i] = 'lowerCase';
    } else if (/[A-Z]/.test(char)) {
      charsTypes[i] = 'upperCase';
    } else if (/[0-9]/.test(char)) {
      charsTypes[i] = 'digit';
    } else {
      charsTypes[i] = 'symbol';
    }

    charsPositions.push(i);
  }

  element.textContent = '';
  var timeout = null;

  (function shuffle(start) {
    var charsArrayCopy = _toConsumableArray(charsArray);

    var charsPositionsLength = charsPositions.length;

    if (start > charsPositionsLength) {
      if (typeof options.onComplete === 'function') {
        options.onComplete(element);
      }

      return;
    }

    for (var _i = Math.max(start, 0); _i < charsPositionsLength; _i += 1) {
      if (_i < start + options.iterations) {
        charsArrayCopy[charsPositions[_i]] = getRandomCharacterForType(charsTypes[charsPositions[_i]]);
      } else {
        charsArrayCopy[charsPositions[_i]] = '';
      }
    }

    element.textContent = charsArrayCopy.join('');
    timeout = setTimeout(function () {
      shuffle(start + 1);
    }, 1000 / options.fps);
  })(-options.iterations);

  return function () {
    clearTimeout(timeout);
    timeout = null;
  };
}

export default shuffleLetters;
